from typing import Dict

from ts_t1_validator.models.enums.validators import ValidatorsEnum
from ts_t1_validator.services.t1 import T1Service
from ts_t1_validator.validators.abstract_validator import AbstractValidator
from ts_t1_validator.validators.campaign_post_validator import CampaignPostValidator
from ts_t1_validator.validators.strategy_post_validator import StrategyPostValidator
from ts_t1_validator.validators.targeting_location_validator import TargetingLocationValidator


class T1Validator:
    def __init__(self, t1_creds: Dict, validator_type="campaign"):
        self.validator = None
        t1_service = T1Service(t1_creds=t1_creds)
        validator_enum = ValidatorsEnum.set(validator_type)
        if validator_enum is ValidatorsEnum.CAMPAIGN:
            self.validator = CampaignPostValidator(t1_service=t1_service)
        elif validator_enum is ValidatorsEnum.STRATEGY:
            self.validator = StrategyPostValidator(t1_service=t1_service)
        elif validator_enum is ValidatorsEnum.TARGETING_LOCATION:
            self.validator = TargetingLocationValidator(t1_service=t1_service)
        else:
            raise NotImplemented()

    def validate(self, *args, **kwargs):
        """
        Proxy validate method to proper class
        :return:
        """
        return self.validator.validate(*args, **kwargs)



from enum import unique

from ts_t1_validator.models.enums.abstract_enum import AbstractEnum


@unique
class TargetingTypeEnum(AbstractEnum):
    INCLUDE = "include"
    EXCLUDE = "exclude"
    UNDEFINED = None



from ts_t1_validator import AbstractValidator, T1Service
from ts_t1_validator.models.dto.targeting_location import TargetingLocationDTO
from ts_t1_validator.models.enums.location_type import LocationTypeEnum
from ts_t1_validator.models.enums.targeting_type import TargetingTypeEnum

from ts_t1_validator.validators.rules.one_of_enum_rule import OneOfEnumRule
from ts_t1_validator.validators.rules.positive_id_list_rule import PositiveIDListRule
from ts_t1_validator.validators.rules.positive_id_rule import PositiveIDRule
from ts_t1_validator.validators.rules.t1_ids_same_check import T1CheckIds
from ts_t1_validator.validators.rules.targeting_location_in_list_rule import TargetingLocationInList
from ts_t1_validator.validators.rules.targeting_locations_in_list_rule import TargetingLocationsInList


class TargetingLocationValidator(AbstractValidator):
    def __init__(self, t1_service: T1Service):
        """
        targeting location validation manager
        :param t1_service: T1Service
        """
        self.rules = list()
        self.errors = list()
        self.json_schema = None
        self.t1_service = t1_service

    def build_rules_set(self, *args, **kwargs):
        """
        Build rules set
        """

        # remove previous rules
        self.rules = list()

        dto = TargetingLocationDTO.fromDict(kwargs.get("dto", {}))

        self.rules.append(PositiveIDListRule(values=dto.included, field_name="Included"))
        self.rules.append(PositiveIDListRule(values=dto.excluded, field_name="Excluded"))
        self.rules.append(OneOfEnumRule(value=dto.location_type, expected_enum=LocationTypeEnum,
                                        field_name="Location Type"))

        self.rules.append(T1CheckIds(include_value_ids=dto.included, exclude_value_ids=dto.excluded,
                                     location_type=dto.location_type))
        self.rules.append(TargetingLocationsInList(values=dto.included,
                                                   location_type=dto.location_type,
                                                   field_name="Included"))

        self.rules.append(TargetingLocationsInList(values=dto.excluded,
                                                   location_type=dto.location_type,
                                                   field_name="Excluded"))
